@@ -28,7 +28,7 @@ fi
 #     )
 #
 # where t is the name of the test, and the '' is the string to search when
-# not cross-compiling (which is empty, because for amd64 everything is
+# not cross-compiling (which is empty, because for x86_64 everything is
 # expected to pass).
 if [ -n "$CROSS" ]; then
     CROSS_XFAIL=.gitlab-ci/cross-xfail-"$CROSS"
@@ -60,8 +60,8 @@ RET=0
 RESULTS_DIR=$(pwd)/results/${TEST_SUITE:-build}
 rm -rf _build
 
-meson _build --native-file=native.file \
-    --wrap-mode=nofallback \
+meson setup _build --native-file=native.file \
+    --wrap-mode=${WRAP_DEBUG:-nofallback} \
     ${CROSS+--cross "$CROSS_FILE"} \
     -D prefix=$(pwd)/install \
     -D libdir=lib \
@@ -73,24 +73,26 @@ meson _build --native-file=native.file \
     -D tests=true \
     -D render-server=true \
     -D render-server-worker=process \
-    -D venus-experimental=true \
+    -D venus=true \
+    -Dtracing=${TRACING_BACKEND:-none} \
     --fatal-meson-warnings \
     ${EXTRA_OPTION} && \
 pushd _build && \
-meson configure && \
-ninja -j ${FDO_CI_CONCURRENT:-4} install || {
+meson compile -j ${FDO_CI_CONCURRENT:-4} || {
     RET=$?
     mkdir -p ${RESULTS_DIR}
     mv -f meson-logs/* ${RESULTS_DIR}/
     popd
     exit ${RET}
 }
+meson install
 
 if [ -n "${TEST_SUITE}" ]; then
-    VRENDTEST_USE_EGL_SURFACELESS=1 ninja -j ${FDO_CI_CONCURRENT:-4} test || RET=$?
+    VRENDTEST_USE_EGL_SURFACELESS=1 meson test --num-processes ${FDO_CI_CONCURRENT:-4} || RET=$?
     mkdir -p ${RESULTS_DIR}
     mv -f meson-logs/testlog.txt ${RESULTS_DIR}/
 fi
 
 popd
 exit ${RET}
+
