@@ -21,19 +21,15 @@
 
 #include "linux/overflow.h"
 
-void _drm_log(const char *fmt, ...);
-#define drm_log(fmt, ...) _drm_log("%s:%d: " fmt, __func__, __LINE__, ##__VA_ARGS__)
+#include "virglrenderer.h"
+
+void _drm_log(enum virgl_log_level_flags level, const char *fmt, ...) __attribute__((format(printf, 2, 3)));
+#define drm_log(fmt, ...) _drm_log(VIRGL_LOG_LEVEL_INFO, "%s:%d: " fmt "\n", __func__, __LINE__, ##__VA_ARGS__)
+#define drm_err(fmt, ...) _drm_log(VIRGL_LOG_LEVEL_ERROR, "%s:%d: " fmt "\n", __func__, __LINE__, ##__VA_ARGS__)
+#define drm_dbg(fmt, ...) _drm_log(VIRGL_LOG_LEVEL_DEBUG, "%s:%d: " fmt "\n", __func__, __LINE__, ##__VA_ARGS__)
 
 #pragma GCC diagnostic pop
 
-#if 0
-#define drm_dbg drm_log
-#else
-#define drm_dbg(fmt, ...)                                                                \
-   do {                                                                                  \
-   } while (0)
-#endif
-
 #define VOID2U64(x) ((uint64_t)(unsigned long)(x))
 #define U642VOID(x) ((void *)(unsigned long)(x))
 
@@ -41,4 +37,10 @@ void _drm_log(const char *fmt, ...);
 #define NSEC_PER_SEC 1000000000ull
 #endif
 
+#if defined(__MINGW32__) || defined(__MINGW64__)
+#  define DRM_ALIGN_4 __attribute__((gcc_struct,__packed__,__aligned__(4)))
+#else
+#  define DRM_ALIGN_4 __attribute__((__packed__, __aligned__(4)))
+#endif
+
 #endif /* DRM_UTIL_H_ */
