@@ -99,6 +99,20 @@ const char *tgsi_semantic_names[TGSI_SEMANTIC_COUNT] =
    "TESSINNER",
    "VERTICESIN",
    "HELPER_INVOCATION",
+   "BASEINSTANCE",
+   "DRAWID",
+   "WORK_DIM",
+   "SUBGROUP_SIZE",
+   "SUBGROUP_INVOCATION",
+   "SUBGROUP_EQ_MASK",
+   "SUBGROUP_GE_MASK",
+   "SUBGROUP_GT_MASK",
+   "SUBGROUP_LE_MASK",
+   "SUBGROUP_LT_MASK",
+   "CS_USER_DATA_AMD",
+   "VIEWPORT_MASK",
+   "TESS_DEFAULT_OUTER_LEVEL",
+   "TESS_DEFAULT_INNER_LEVEL"
 };
 
 const char *tgsi_texture_names[TGSI_TEXTURE_COUNT] =
