@@ -20,10 +20,10 @@
 
 #include "c11/threads.h"
 #include "util/hash_table.h"
+#include "util/list.h"
 #include "util/macros.h"
 #include "util/os_file.h"
 #include "util/os_misc.h"
-#include "util/u_double_list.h"
 #include "util/u_math.h"
 #include "util/u_memory.h"
 #include "util/u_pointer.h"
@@ -95,6 +95,7 @@ typedef uint64_t vkr_object_id;
 
 enum vkr_debug_flags {
    VKR_DEBUG_VALIDATE = 1 << 0,
+   VKR_DEBUG_UDMABUF = 1 << 1,
 };
 
 /* base class for all objects */
@@ -253,6 +254,7 @@ vkr_is_recognized_object_type(VkObjectType type)
    case VK_OBJECT_TYPE_DESCRIPTOR_UPDATE_TEMPLATE:
       return true;
    default:
+      vkr_log("unrecognized object type %u", type);
       return false;
    }
 }
@@ -316,4 +318,11 @@ vkr_region_make_relative(const struct vkr_region *region)
    };
 }
 
+static inline bool
+vkr_seqno_ge(uint32_t a, uint32_t b)
+{
+   /* a >= b, but deal with wrapping as well */
+   return (a - b) <= INT32_MAX;
+}
+
 #endif /* VKR_COMMON_H */
