@@ -153,7 +153,7 @@ vkr_context_submit_cmd(struct vkr_context *ctx, const void *buffer, size_t size)
       return false;
    }
 
-   vkr_cs_decoder_set_stream(&ctx->decoder, buffer, size);
+   vkr_cs_decoder_set_buffer_stream(&ctx->decoder, buffer, size);
 
    while (vkr_cs_decoder_has_command(&ctx->decoder)) {
       vn_dispatch_command(&ctx->dispatch);
@@ -237,6 +237,27 @@ vkr_context_import_resource_internal(struct vkr_context *ctx,
    return true;
 }
 
+static bool
+vkr_context_import_resource_from_shm(struct vkr_context *ctx,
+                                     uint32_t res_id,
+                                     uint64_t blob_size,
+                                     int fd)
+{
+   assert(!vkr_context_get_resource(ctx, res_id));
+
+   void *mmap_ptr = mmap(NULL, blob_size, PROT_WRITE | PROT_READ, MAP_SHARED, fd, 0);
+   if (mmap_ptr == MAP_FAILED)
+      return false;
+
+   if (!vkr_context_import_resource_internal(ctx, res_id, blob_size,
+                                             VIRGL_RESOURCE_FD_SHM, -1, mmap_ptr)) {
+      munmap(mmap_ptr, blob_size);
+      return false;
+   }
+
+   return true;
+}
+
 static bool
 vkr_context_create_resource_from_shm(struct vkr_context *ctx,
                                      uint32_t res_id,
@@ -340,6 +361,9 @@ vkr_context_import_resource(struct vkr_context *ctx,
                             int fd,
                             uint64_t size)
 {
+   if (fd_type == VIRGL_RESOURCE_FD_SHM)
+      return vkr_context_import_resource_from_shm(ctx, res_id, size, fd);
+
    return vkr_context_import_resource_internal(ctx, res_id, size, fd_type, fd, NULL);
 }
 
@@ -350,40 +374,32 @@ vkr_context_destroy_resource(struct vkr_context *ctx, uint32_t res_id)
    if (!res)
       return;
 
-   vkr_cs_encoder_check_stream(&ctx->encoder, res);
-
-   mtx_lock(&ctx->ring_mutex);
-   struct vkr_ring *ring, *ring_tmp;
-   LIST_FOR_EACH_ENTRY_SAFE (ring, ring_tmp, &ctx->rings, head) {
-      vkr_cs_encoder_check_stream(&ring->encoder, res);
-
-      if (ring->resource != res)
-         continue;
-
+   if (!vkr_cs_encoder_check_stream(&ctx->encoder, res))
       vkr_context_set_fatal(ctx);
 
-      mtx_unlock(&ctx->ring_mutex);
-      vkr_ring_stop(ring);
-      mtx_lock(&ctx->ring_mutex);
+   mtx_lock(&ctx->ring_mutex);
+   list_for_each_entry_safe (struct vkr_ring, ring, &ctx->rings, head) {
+      if (ring->resource == res ||
+          !vkr_cs_decoder_check_stream(&ring->decoder, res) ||
+          !vkr_cs_encoder_check_stream(&ring->encoder, res)) {
+         vkr_context_set_fatal(ctx);
 
-      vkr_ring_destroy(ring);
+         mtx_unlock(&ctx->ring_mutex);
+         vkr_ring_stop(ring);
+         mtx_lock(&ctx->ring_mutex);
+
+         vkr_ring_destroy(ring);
+      }
    }
    mtx_unlock(&ctx->ring_mutex);
 
    vkr_context_remove_resource(ctx, res_id);
 }
 
-static inline bool
-vkr_seqno_ge(uint32_t a, uint32_t b)
-{
-   /* a >= b, but deal with wrapping as well */
-   return (a - b) <= INT32_MAX;
-}
-
 void
 vkr_context_on_ring_seqno_update(struct vkr_context *ctx,
                                  uint64_t ring_id,
-                                 uint64_t ring_seqno)
+                                 uint32_t ring_seqno)
 {
    mtx_lock(&ctx->wait_ring.mutex);
    if (ctx->wait_ring.id == ring_id && vkr_seqno_ge(ring_seqno, ctx->wait_ring.seqno))
@@ -391,6 +407,31 @@ vkr_context_on_ring_seqno_update(struct vkr_context *ctx,
    mtx_unlock(&ctx->wait_ring.mutex);
 }
 
+bool
+vkr_context_get_wait_ring_seqno(struct vkr_context *ctx,
+                                uint64_t ring_id,
+                                uint32_t *out_seqno)
+{
+   bool wait_ring = false;
+   mtx_lock(&ctx->wait_ring.mutex);
+   if (ctx->wait_ring.id == ring_id) {
+      wait_ring = true;
+      *out_seqno = ctx->wait_ring.seqno;
+   }
+   mtx_unlock(&ctx->wait_ring.mutex);
+   return wait_ring;
+}
+
+void
+vkr_context_on_ring_fatal(struct vkr_context *ctx)
+{
+   vkr_context_set_fatal(ctx);
+
+   mtx_lock(&ctx->wait_ring.mutex);
+   cnd_signal(&ctx->wait_ring.cond);
+   mtx_unlock(&ctx->wait_ring.mutex);
+}
+
 bool
 vkr_context_wait_ring_seqno(struct vkr_context *ctx,
                             struct vkr_ring *ring,
@@ -403,7 +444,8 @@ vkr_context_wait_ring_seqno(struct vkr_context *ctx,
    mtx_lock(&ctx->wait_ring.mutex);
    ctx->wait_ring.id = ring->id;
    ctx->wait_ring.seqno = ring_seqno;
-   while (ok && !vkr_seqno_ge(vkr_ring_load_head(ring), ring_seqno)) {
+   while (!vkr_context_get_fatal(ctx) && ok &&
+          !vkr_seqno_ge(vkr_ring_load_head(ring), ring_seqno)) {
       ok = cnd_wait(&ctx->wait_ring.cond, &ctx->wait_ring.mutex) == thrd_success;
    }
    ctx->wait_ring.id = 0;
@@ -551,8 +593,7 @@ vkr_context_destroy(struct vkr_context *ctx)
    /* TODO Move the entire teardown process to a separate thread so that the main thread
     * cannot get blocked by the vkDeviceWaitIdle upon device destruction.
     */
-   struct vkr_ring *ring, *ring_tmp;
-   LIST_FOR_EACH_ENTRY_SAFE (ring, ring_tmp, &ctx->rings, head) {
+   list_for_each_entry_safe (struct vkr_ring, ring, &ctx->rings, head) {
       vkr_ring_stop(ring);
       vkr_ring_destroy(ring);
    }
@@ -567,7 +608,7 @@ vkr_context_destroy(struct vkr_context *ctx)
       vkr_log("destroying context %d (%s) with a valid instance", ctx->ctx_id,
               vkr_context_get_name(ctx));
 
-      vkr_instance_destroy(ctx, ctx->instance);
+      vkr_instance_destroy(ctx, ctx->instance, false);
    }
 
    _mesa_hash_table_destroy(ctx->resource_table, vkr_context_free_resource);
@@ -631,6 +672,12 @@ vkr_context_create(uint32_t ctx_id,
    if (VKR_DEBUG(VALIDATE))
       ctx->validate_level = VKR_CONTEXT_VALIDATE_FULL;
 
+#ifdef ENABLE_RENDER_SERVER_WORKER_THREAD
+   ctx->on_worker_thread = true;
+#else
+   ctx->on_worker_thread = false;
+#endif
+
    if (!vkr_context_wait_ring_init(ctx))
       goto err_ctx_wait_ring_init;
 
@@ -649,7 +696,9 @@ vkr_context_create(uint32_t ctx_id,
    if (!ctx->resource_table)
       goto err_ctx_resource_table;
 
-   vkr_cs_decoder_init(&ctx->decoder, &ctx->cs_fatal_error, ctx->object_table);
+   if (vkr_cs_decoder_init(&ctx->decoder, ctx))
+      goto err_cs_decoder_init;
+
    if (vkr_cs_encoder_init(&ctx->encoder, &ctx->cs_fatal_error))
       goto err_cs_encoder_init;
 
@@ -665,6 +714,8 @@ vkr_context_create(uint32_t ctx_id,
 err_ctx_ring_mutex:
    vkr_cs_encoder_fini(&ctx->encoder);
 err_cs_encoder_init:
+   vkr_cs_decoder_fini(&ctx->decoder);
+err_cs_decoder_init:
    _mesa_hash_table_destroy(ctx->resource_table, vkr_context_free_resource);
 err_ctx_resource_table:
    mtx_destroy(&ctx->resource_mutex);
