@@ -26,6 +26,8 @@ void
 vkr_context_init_instance_dispatch(struct vkr_context *ctx);
 
 void
-vkr_instance_destroy(struct vkr_context *ctx, struct vkr_instance *instance);
+vkr_instance_destroy(struct vkr_context *ctx,
+                     struct vkr_instance *instance,
+                     bool destroy_vk);
 
 #endif /* VKR_INSTANCE_H */
