@@ -11,6 +11,38 @@
 #include "vkr_device.h"
 #include "vkr_instance.h"
 
+#ifdef HAVE_LINUX_UDMABUF_H
+#include <fcntl.h>
+
+static int udmabuf_dev_fd = -1;
+
+static void
+vkr_udmabuf_init_once(void)
+{
+   udmabuf_dev_fd = open("/dev/udmabuf", O_RDWR);
+   if (udmabuf_dev_fd < 0) {
+      vkr_log("failed to open udmabuf device: %s", strerror(errno));
+   }
+}
+
+static inline int
+vkr_physical_device_get_udmabuf_dev_fd(void)
+{
+   static once_flag udmabuf_once_flag = ONCE_FLAG_INIT;
+   call_once(&udmabuf_once_flag, vkr_udmabuf_init_once);
+   return udmabuf_dev_fd;
+}
+
+#else /* HAVE_LINUX_UDMABUF_H */
+
+static inline int
+vkr_physical_device_get_udmabuf_dev_fd(void)
+{
+   return -1;
+}
+
+#endif /* HAVE_LINUX_UDMABUF_H */
+
 #ifdef ENABLE_MINIGBM_ALLOCATION
 #include <gbm.h>
 #ifdef MINIGBM
@@ -57,11 +89,11 @@ void
 vkr_physical_device_destroy(struct vkr_context *ctx,
                             struct vkr_physical_device *physical_dev)
 {
-   struct vkr_device *dev, *tmp;
-   LIST_FOR_EACH_ENTRY_SAFE (dev, tmp, &physical_dev->devices, base.track_head)
-      vkr_device_destroy(ctx, dev);
+   list_for_each_entry_safe (struct vkr_device, dev, &physical_dev->devices, base.track_head)
+      vkr_device_destroy(ctx, dev, false);
 
    free(physical_dev->extensions);
+   free(physical_dev->queue_family_properties);
 
    vkr_context_remove_object(ctx, &physical_dev->base);
 }
@@ -193,6 +225,9 @@ vkr_physical_device_init_memory_properties(struct vkr_physical_device *physical_
    if (!physical_dev->is_dma_buf_fd_export_supported &&
        !physical_dev->is_opaque_fd_export_supported)
       physical_dev->gbm_device = vkr_physical_device_get_gbm_device();
+
+   physical_dev->udmabuf_dev_fd =
+      VKR_DEBUG(UDMABUF) ? vkr_physical_device_get_udmabuf_dev_fd() : -1;
 }
 
 static void
@@ -253,7 +288,7 @@ vkr_physical_device_init_extensions(struct vkr_physical_device *physical_dev,
          physical_dev->KHR_external_fence_fd = false;
    }
 
-   physical_dev->extensions = exts;
+   physical_dev->extensions = realloc(exts, sizeof(*exts) * advertised_count);
    physical_dev->extension_count = advertised_count;
 }
 
@@ -275,6 +310,25 @@ vkr_physical_device_init_proc_table(struct vkr_physical_device *physical_dev,
                                            &physical_dev->proc_table);
 }
 
+static void
+vkr_physical_device_init_queue_family_properties(struct vkr_physical_device *physical_dev)
+{
+   VkPhysicalDevice handle = physical_dev->base.handle.physical_device;
+
+   VkQueueFamilyProperties *props;
+   uint32_t count;
+   vkGetPhysicalDeviceQueueFamilyProperties(handle, &count, NULL);
+
+   props = malloc(sizeof(*props) * count);
+   if (!props)
+      return;
+
+   vkGetPhysicalDeviceQueueFamilyProperties(handle, &count, props);
+
+   physical_dev->queue_family_property_count = count;
+   physical_dev->queue_family_properties = props;
+}
+
 static void
 vkr_dispatch_vkEnumeratePhysicalDevices(struct vn_dispatch_context *dispatch,
                                         struct vn_command_vkEnumeratePhysicalDevices *args)
@@ -339,6 +393,7 @@ vkr_dispatch_vkEnumeratePhysicalDevices(struct vn_dispatch_context *dispatch,
       vkr_physical_device_init_extensions(physical_dev, instance);
       vkr_physical_device_init_memory_properties(physical_dev);
       vkr_physical_device_init_id_properties(physical_dev);
+      vkr_physical_device_init_queue_family_properties(physical_dev);
 
       list_inithead(&physical_dev->devices);
 
@@ -353,6 +408,7 @@ vkr_dispatch_vkEnumeratePhysicalDevices(struct vn_dispatch_context *dispatch,
          if (!physical_dev)
             break;
          free(physical_dev->extensions);
+         free(physical_dev->queue_family_properties);
          vkr_context_remove_object(ctx, &physical_dev->base);
          instance->physical_devices[i] = NULL;
       }
@@ -660,6 +716,20 @@ vkr_dispatch_vkGetPhysicalDeviceCalibrateableTimeDomainsEXT(
       args->physicalDevice, args->pTimeDomainCount, args->pTimeDomains);
 }
 
+static void
+vkr_dispatch_vkGetPhysicalDeviceFragmentShadingRatesKHR(
+   UNUSED struct vn_dispatch_context *ctx,
+   struct vn_command_vkGetPhysicalDeviceFragmentShadingRatesKHR *args)
+{
+   struct vkr_physical_device *physical_dev =
+      vkr_physical_device_from_handle(args->physicalDevice);
+   struct vn_physical_device_proc_table *vk = &physical_dev->proc_table;
+
+   vn_replace_vkGetPhysicalDeviceFragmentShadingRatesKHR_args_handle(args);
+   args->ret = vk->GetPhysicalDeviceFragmentShadingRatesKHR(
+      args->physicalDevice, args->pFragmentShadingRateCount, args->pFragmentShadingRates);
+}
+
 void
 vkr_context_init_physical_device_dispatch(struct vkr_context *ctx)
 {
@@ -711,4 +781,8 @@ vkr_context_init_physical_device_dispatch(struct vkr_context *ctx)
       vkr_dispatch_vkGetPhysicalDeviceExternalFenceProperties;
    dispatch->dispatch_vkGetPhysicalDeviceCalibrateableTimeDomainsEXT =
       vkr_dispatch_vkGetPhysicalDeviceCalibrateableTimeDomainsEXT;
+
+   /* VK_KHR_fragment_shading_rate */
+   dispatch->dispatch_vkGetPhysicalDeviceFragmentShadingRatesKHR =
+      vkr_dispatch_vkGetPhysicalDeviceFragmentShadingRatesKHR;
 }
