@@ -52,6 +52,8 @@ enum virgl_debug_flags {
    dbg_allow_guest_override = 1 << 16,
    dbg_feature_use = 1 << 17,
    dbg_khr = 1 << 18,
+   dbg_d3d = 1 << 19,
+   dbg_dump_cmd_streams = 1 << 20,
 };
 
 const char *vrend_get_comand_name(enum virgl_context_cmd cmd);
@@ -71,14 +73,6 @@ unsigned vrend_debug(const struct vrend_context *ctx, enum virgl_debug_flags fla
 
 void vrend_debug_add_flag(enum virgl_debug_flags flag);
 
-static inline void vrend_printf(const char *fmt, ...)
-{
-   va_list va;
-   va_start(va, fmt);
-   virgl_logv(fmt, va);
-   va_end(va);
-}
-
 #ifdef NDEBUG
 #define VREND_DEBUG_ENABLED (false)
 #else
@@ -89,7 +83,7 @@ static inline void vrend_printf(const char *fmt, ...)
    if (VREND_DEBUG_ENABLED && vrend_debug(ctx, flag)) \
       do { \
             vrend_print_context_name(ctx); \
-            vrend_printf(__VA_ARGS__); \
+            virgl_debug(__VA_ARGS__); \
       } while (0)
 
 #define VREND_DEBUG_EXT(flag, ctx, X) \
@@ -102,7 +96,7 @@ static inline void vrend_printf(const char *fmt, ...)
 #define VREND_DEBUG_NOCTX(flag, ctx, ...) \
    if (VREND_DEBUG_ENABLED && vrend_debug(ctx, flag)) \
       do { \
-            vrend_printf(__VA_ARGS__); \
+            virgl_debug(__VA_ARGS__); \
       } while (0)
 
 #endif
