@@ -33,6 +33,7 @@
 #include <string.h>
 #include <xf86drm.h>
 #include <unistd.h>
+#include <sys/mman.h>
 
 #include "util/u_math.h"
 #include "util/u_memory.h"
@@ -102,7 +103,9 @@ static const struct format_conversion conversions[] = {
     { GBM_FORMAT_ARGB8888, VIRGL_FORMAT_B8G8R8A8_UNORM },
     { GBM_FORMAT_XRGB8888, VIRGL_FORMAT_B8G8R8X8_UNORM },
     { GBM_FORMAT_ABGR2101010, VIRGL_FORMAT_R10G10B10A2_UNORM },
+    { GBM_FORMAT_XRGB2101010, VIRGL_FORMAT_B10G10R10X2_UNORM },
     { GBM_FORMAT_ABGR16161616F, VIRGL_FORMAT_R16G16B16A16_FLOAT },
+    { GBM_FORMAT_XBGR16161616F, VIRGL_FORMAT_R16G16B16X16_FLOAT },
     { GBM_FORMAT_NV12, VIRGL_FORMAT_NV12 },
     { GBM_FORMAT_ABGR8888, VIRGL_FORMAT_R8G8B8A8_UNORM},
     { GBM_FORMAT_XBGR8888, VIRGL_FORMAT_R8G8B8X8_UNORM},
@@ -329,7 +332,7 @@ int virgl_gbm_convert_format(uint32_t *virgl_format, uint32_t *gbm_format)
 int virgl_gbm_transfer(struct gbm_bo *bo, uint32_t direction, const struct iovec *iovecs,
                        uint32_t num_iovecs, const struct vrend_transfer_info *info)
 {
-   void *map_data;
+   void *map_data = NULL;
    uint32_t guest_plane_offset, guest_stride0, host_map_stride0;
 
    uint32_t width = gbm_bo_get_width(bo);
@@ -350,7 +353,7 @@ int virgl_gbm_transfer(struct gbm_bo *bo, uint32_t direction, const struct iovec
       map_flags |= GBM_BO_TRANSFER_READ;
 
    void *addr = gbm_bo_map(bo, 0, 0, width, height, map_flags, &host_map_stride0, &map_data);
-   if (!addr)
+   if (!addr || addr == MAP_FAILED)
       return -1;
 
    guest_plane_offset = info->offset;
@@ -516,7 +519,45 @@ err_close:
    query->out_num_fds = 0;
    return ret;
 }
+
+uint32_t virgl_gbm_get_map_info(struct gbm_bo *bo) {
+   uint32_t map_info;
+#if defined(MINIGBM) && defined(MINIGBM_HAS_GBM_BO_GET_MAP_INFO)
+      enum gbm_bo_map_cache_mode cache_mode = gbm_bo_get_map_info(bo);
+      switch(cache_mode) {
+      /* case GBM_BO_MAP_CACHE_NONE: */
+      /*    map_info = VIRGL_RENDERER_MAP_CACHE_NONE; */
+      /*    break; */
+      case GBM_BO_MAP_CACHE_CACHED:
+         map_info = VIRGL_RENDERER_MAP_CACHE_CACHED;
+         break;
+      /* case GBM_BO_MAP_CACHE_UNCACHED: */
+      /*    map_info = VIRGL_RENDERER_MAP_CACHE_UNCACHED; */
+      /*    break; */
+      case GBM_BO_MAP_CACHE_WC:
+      default:
+         map_info = VIRGL_RENDERER_MAP_CACHE_WC;
+         break;
+      }
+#else
+   /* That GPU allocations on all intel devices are CACHED is famously untrue
+    * on Intel devices that don't have a shared last-level-cache architecture to
+    * ensure memory coherency between CPU/GPU cores (e.g. GLK). Thus, mistakenly
+    * reporting CACHED mapping type would cause stale cached CPU access to GPU
+    * writes that are resident only in system memory.
+    *
+    * Still this heuristic is retained as a fallback for the missing minigbm API
+    */
+   if (!strcmp(gbm_device_get_backend_name(gbm->device), "i915"))
+      map_info = VIRGL_RENDERER_MAP_CACHE_CACHED;
+   else
+      map_info = VIRGL_RENDERER_MAP_CACHE_WC;
+
+   (void)bo;
 #endif
+   return map_info;
+}
+#endif // #ifdef ENABLE_MINIGBM_ALLOCATION
 
 int virgl_gbm_export_fd(struct gbm_device *gbm, uint32_t handle, int32_t *out_fd)
 {
