@@ -28,7 +28,7 @@
 #include <virglrenderer.h>
 #include "virgl_hw.h"
 #include "vrend_iov.h"
-#include "pipe/p_format.h"
+#include "util/u_formats.h"
 #include "testvirgl_encode.h"
 #include "virgl_protocol.h"
 #include "util/u_memory.h"
@@ -40,7 +40,7 @@ START_TEST(virgl_test_overlap_obj_id)
     int ret;
     struct virgl_context ctx;
     int ctx_handle = 1;
-    ret = testvirgl_init_ctx_cmdbuf(&ctx);
+    ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
     ck_assert_int_eq(ret, 0);
 
     /* set blend state */
@@ -85,7 +85,7 @@ START_TEST(virgl_test_clear)
     int ret;
     int i;
 
-    ret = testvirgl_init_ctx_cmdbuf(&ctx);
+    ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
     ck_assert_int_eq(ret, 0);
 
     /* init and create simple 2D resource */
@@ -157,7 +157,7 @@ START_TEST(virgl_test_blit_simple)
     int ret;
     int i;
 
-    ret = testvirgl_init_ctx_cmdbuf(&ctx);
+    ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
     ck_assert_int_eq(ret, 0);
 
     /* init and create simple 2D resource */
@@ -274,7 +274,7 @@ START_TEST(virgl_test_render_simple)
     int ret;
     int tw = 300, th = 300;
 
-    ret = testvirgl_init_ctx_cmdbuf(&ctx);
+    ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
     ck_assert_int_eq(ret, 0);
 
     /* init and create simple 2D resource */
@@ -442,12 +442,12 @@ START_TEST(virgl_test_render_simple)
         virgl_encoder_draw_vbo(&ctx, &info);
     }
 
-    testvirgl_ctx_send_cmdbuf(&ctx);
+    ret = testvirgl_ctx_send_cmdbuf(&ctx);
+    ck_assert_int_eq(ret, 0);
 
     /* create a fence */
     testvirgl_reset_fence();
-    ret = virgl_renderer_create_fence(1, ctx.ctx_id);
-    ck_assert_int_eq(ret, 0);
+    virgl_renderer_create_fence(1, ctx.ctx_id);
 
     do {
         int fence;
@@ -492,6 +492,61 @@ START_TEST(virgl_test_render_simple)
 }
 END_TEST
 
+static void virgl_test_bind_images_shader(int first_layer, int last_layer, int expected_error)
+{
+    struct virgl_context ctx;
+    struct virgl_resource res;
+    struct virgl_surface surf = {0};
+    int ret;
+    int tw = 300, th = 300;
+
+    ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
+    ck_assert_int_eq(ret, 0);
+
+    /* init and create simple 2D resource */
+    ret = testvirgl_create_backed_simple_2d_res(&res, 1, tw, th);
+    ck_assert_int_eq(ret, 0);
+
+    /* attach resource to context */
+    virgl_renderer_ctx_attach_resource(ctx.ctx_id, res.handle);
+
+    /* set the framebuffer state */
+    struct pipe_framebuffer_state fb_state = {
+       .nr_cbufs = 1,
+       .zsbuf = NULL,
+       .cbufs[0] = &surf.base
+    };
+    virgl_encoder_set_framebuffer_state(&ctx, &fb_state);
+
+    /* Create shader images */
+    struct vrend_image_view images[1] = {0};
+    images[0].u.tex.first_layer = first_layer;
+    images[0].u.tex.last_layer = last_layer;
+    virgl_encoder_set_shader_images(&ctx, PIPE_SHADER_FRAGMENT, 1, 1, images, res.handle);
+
+    ret = testvirgl_ctx_send_cmdbuf(&ctx);
+    ck_assert_int_eq(ret, expected_error);
+
+    /* cleanup */
+    virgl_renderer_ctx_detach_resource(ctx.ctx_id, res.handle);
+
+    testvirgl_destroy_backed_res(&res);
+
+    testvirgl_fini_ctx_cmdbuf(&ctx);
+}
+
+START_TEST(virgl_test_bind_images_shader_pass)
+{
+    virgl_test_bind_images_shader(0, 0, 0);
+}
+END_TEST
+
+START_TEST(virgl_test_bind_images_shader_fail_layers)
+{
+    virgl_test_bind_images_shader(1, 0, EINVAL);
+}
+END_TEST
+
 /* create a resource - clear it to a color, render something */
 START_TEST(virgl_test_render_geom_simple)
 {
@@ -509,7 +564,7 @@ START_TEST(virgl_test_render_geom_simple)
     int ret;
     int tw = 300, th = 300;
 
-    ret = testvirgl_init_ctx_cmdbuf(&ctx);
+    ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
     ck_assert_int_eq(ret, 0);
 
     /* Geometry shader are only available since GLSL 150 */
@@ -778,7 +833,7 @@ START_TEST(virgl_test_render_xfb)
     int ret;
     int tw = 300, th = 300;
 
-    ret = testvirgl_init_ctx_cmdbuf(&ctx);
+    ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
     ck_assert_int_eq(ret, 0);
 
     /* init and create simple 2D resource */
@@ -1009,7 +1064,7 @@ START_TEST(virgl_test_large_shader)
    struct virgl_context ctx;
    int ctx_handle = 1;
    int fs_handle;
-   ret = testvirgl_init_ctx_cmdbuf(&ctx);
+   ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
    ck_assert_int_eq(ret, 0);
 
    /* create large fragment shader */
@@ -1032,7 +1087,7 @@ END_TEST
 START_TEST(virgl_test_set_viewport_state)
 {
    struct virgl_context ctx;
-   int ret = testvirgl_init_ctx_cmdbuf(&ctx);
+   int ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
    ck_assert_int_eq(ret, 0);
 
    struct pipe_viewport_state vp[PIPE_MAX_VIEWPORTS];
@@ -1095,7 +1150,7 @@ START_TEST(virgl_test_set_viewport_state)
 START_TEST(virgl_decode_set_scissor_state)
 {
    struct virgl_context ctx;
-   int ret = testvirgl_init_ctx_cmdbuf(&ctx);
+   int ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
    ck_assert_int_eq(ret, 0);
 
    struct pipe_scissor_state ss[PIPE_MAX_VIEWPORTS + 1] = {0};
@@ -1143,7 +1198,7 @@ END_TEST
 START_TEST(virgl_decode_set_constant_buffer)
 {
    struct virgl_context ctx;
-   int ret = testvirgl_init_ctx_cmdbuf(&ctx);
+   int ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
    ck_assert_int_eq(ret, 0);
 
    uint8_t constant_buffer[10] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };
@@ -1174,7 +1229,7 @@ END_TEST
 START_TEST(virgl_decode_bind_sampler_states)
 {
    struct virgl_context ctx;
-   int ret = testvirgl_init_ctx_cmdbuf(&ctx);
+   int ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
    ck_assert_int_eq(ret, 0);
 
    uint32_t handles[10] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };
@@ -1198,7 +1253,7 @@ END_TEST
 START_TEST(virgl_test_encode_sampler_view)
 {
    struct virgl_context ctx;
-   int ret = testvirgl_init_ctx_cmdbuf(&ctx);
+   int ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
    ck_assert_int_eq(ret, 0);
 
    struct pipe_sampler_view state = {
@@ -1254,6 +1309,101 @@ START_TEST(virgl_test_encode_sampler_view)
    ret = testvirgl_ctx_send_cmdbuf(&ctx);
    ck_assert_int_eq(ret, EINVAL);
 
+   testvirgl_destroy_backed_res(&res);
+   testvirgl_destroy_backed_res(&res2);
+
+   testvirgl_fini_ctx_cmdbuf(&ctx);
+}
+END_TEST
+
+START_TEST(virgl_test_encode_sampler_view_fail_layers)
+{
+   struct virgl_context ctx;
+   int ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
+   ck_assert_int_eq(ret, 0);
+
+   struct pipe_sampler_view state1, state2;
+
+   memset(&state1, 0, sizeof(state1));
+   memset(&state2, 0, sizeof(state2));
+
+   struct virgl_resource res;
+   struct virgl_resource res2;
+
+   /* init and create simple 2D resource */
+   ret = testvirgl_create_backed_simple_2d_res(&res, 1, 50, 50);
+   ck_assert_int_eq(ret, 0);
+
+   /* init and create simple 2D resource */
+   ret = testvirgl_create_backed_simple_2d_res(&res2, 2, 50, 50);
+   ck_assert_int_eq(ret, 0);
+
+   /* attach resource to context */
+   virgl_renderer_ctx_attach_resource(ctx.ctx_id, res.handle);
+
+   /* Try to hit limits of layers */
+   state1.format = VIRGL_FORMAT_B8G8R8A8_UNORM;
+   state1.u.tex.first_layer = 2;
+   state1.u.tex.last_layer = 0;
+
+   ret = virgl_encode_sampler_view(&ctx, res.handle,
+                                   &res,
+                                   &state1);
+   ck_assert_int_eq(ret, 0);
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   /* Try to hit limits of levels */
+   state2.format = VIRGL_FORMAT_B8G8R8A8_UNORM;
+   state2.u.tex.first_level = 2;
+   state2.u.tex.last_level = 0;
+
+   ret = virgl_encode_sampler_view(&ctx, res.handle,
+                                   &res,
+                                   &state2);
+   ck_assert_int_eq(ret, 0);
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   testvirgl_destroy_backed_res(&res);
+   testvirgl_destroy_backed_res(&res2);
+
+   testvirgl_fini_ctx_cmdbuf(&ctx);
+}
+END_TEST
+
+START_TEST(virgl_test_encode_sampler_view_texture_buffer)
+{
+   struct virgl_context ctx;
+   int ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
+   ck_assert_int_eq(ret, 0);
+
+   struct pipe_sampler_view state1;
+
+   memset(&state1, 0, sizeof(state1));
+
+   struct virgl_resource res = {0};
+
+   /* init and create simple 2D resource */
+   ret = testvirgl_create_backed_simple_buffer(&res, 1, 50, PIPE_BIND_SAMPLER_VIEW);
+   ck_assert_int_eq(ret, 0);
+
+   state1.format = PIPE_FORMAT_R8_UNORM;
+   state1.u.buf.first_element = 0;
+   state1.u.buf.last_element = 2;
+
+   /* attach resource to context */
+   virgl_renderer_ctx_attach_resource(ctx.ctx_id, res.handle);
+
+   ret = virgl_encode_sampler_view(&ctx, res.handle,
+                                   &res,
+                                   &state1);
+   ck_assert_int_eq(ret, 0);
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, 0);
+
+   testvirgl_destroy_backed_res(&res);
+
    testvirgl_fini_ctx_cmdbuf(&ctx);
 }
 END_TEST
@@ -1265,7 +1415,7 @@ static void test_create_surface(enum pipe_format format, int expected_error)
    struct virgl_surface surf;
    int ret;
 
-   ret = testvirgl_init_ctx_cmdbuf(&ctx);
+   ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
    ck_assert_int_eq(ret, 0);
 
    /* init and create simple 2D resource */
@@ -1291,6 +1441,14 @@ static void test_create_surface(enum pipe_format format, int expected_error)
    /* cleanup */
    virgl_renderer_ctx_detach_resource(ctx.ctx_id, res.handle);
 
+   /* Check usage with illegal resource */
+   ret = virgl_encoder_create_surface(&ctx, surf.handle, &res, &surf.base);
+   ck_assert_int_eq(ret, 0);
+
+   /* submit the cmd stream */
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
    testvirgl_destroy_backed_res(&res);
 
    testvirgl_fini_ctx_cmdbuf(&ctx);
@@ -1308,6 +1466,509 @@ START_TEST(virgl_test_create_surface_fail_format)
 }
 END_TEST
 
+START_TEST(virgl_test_create_surface_fail_layers)
+{
+   struct virgl_context ctx;
+   struct virgl_resource res;
+   struct virgl_surface surf;
+   int ret;
+
+   ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
+   ck_assert_int_eq(ret, 0);
+
+   /* init and create simple 2D resource */
+   ret = testvirgl_create_backed_simple_2d_res(&res, 1, 50, 50);
+   ck_assert_int_eq(ret, 0);
+
+   /* attach resource to context */
+   virgl_renderer_ctx_attach_resource(ctx.ctx_id, res.handle);
+
+   /* create a surface for the resource */
+   memset(&surf, 0, sizeof(surf));
+   surf.base.format = VIRGL_FORMAT_B8G8R8A8_UNORM;
+   surf.handle = 1;
+   surf.base.texture = &res.base;
+   surf.base.u.tex.first_layer = 2;
+   surf.base.u.tex.last_layer = 0;
+
+   ret = virgl_encoder_create_surface(&ctx, surf.handle, &res, &surf.base);
+   ck_assert_int_eq(ret, 0);
+
+   /* submit the cmd stream */
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   /* cleanup */
+   virgl_renderer_ctx_detach_resource(ctx.ctx_id, res.handle);
+
+   testvirgl_destroy_backed_res(&res);
+
+   testvirgl_fini_ctx_cmdbuf(&ctx);
+}
+END_TEST
+
+START_TEST(virgl_test_query)
+{
+   struct virgl_context ctx;
+   struct virgl_resource res = {0};
+   struct virgl_resource qbo = {0};
+   int ret;
+
+   ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
+   ck_assert_int_eq(ret, 0);
+
+   /* init and create simple 2D resource */
+   ret = testvirgl_create_backed_simple_buffer(&res, 1, 50, VIRGL_BIND_CUSTOM);
+   ck_assert_int_eq(ret, 0);
+
+   ret = testvirgl_create_backed_simple_buffer(&qbo, 2, 50, VIRGL_BIND_QUERY_BUFFER);
+   ck_assert_int_eq(ret, 0);
+
+   /* attach resource to context */
+   virgl_renderer_ctx_attach_resource(ctx.ctx_id, res.handle);
+   virgl_renderer_ctx_attach_resource(ctx.ctx_id, qbo.handle);
+
+   virgl_encoder_create_query(&ctx, 2000, 0, &res, 0);
+   virgl_encoder_begin_query(&ctx, 2000);
+   virgl_encoder_end_query(&ctx, 2000);
+   virgl_encoder_get_query_result(&ctx, 2000, 1);
+   virgl_encoder_get_query_result_qbo(&ctx, 2000, qbo.handle, 1, 4, 100, 0);
+
+   /* submit the cmd stream */
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, 0);
+
+   virgl_renderer_ctx_detach_resource(ctx.ctx_id, res.handle);
+
+   /* Try with invalid resource */
+   virgl_encoder_create_query(&ctx, 100, 0, &res, 0);
+
+   /* submit the cmd stream */
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   virgl_encoder_begin_query(&ctx, 100);
+
+   /* submit the cmd stream */
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   virgl_encoder_end_query(&ctx, 100);
+
+   /* submit the cmd stream */
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   virgl_encoder_get_query_result(&ctx, 100, 1);
+
+   /* submit the cmd stream */
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   virgl_encoder_get_query_result_qbo(&ctx, 100, qbo.handle, 1, 4, 100, 0);
+
+   /* submit the cmd stream */
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   /* cleanup */
+   testvirgl_destroy_backed_res(&res);
+
+   testvirgl_fini_ctx_cmdbuf(&ctx);
+}
+END_TEST
+
+static void test_vertex_elements(int ve_num, int expected_error)
+{
+   struct virgl_context ctx;
+   struct pipe_vertex_element ve[PIPE_MAX_ATTRIBS+1];
+
+   int ret;
+
+   ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
+   ck_assert_int_eq(ret, 0);
+
+   /* create vertex elements */
+   memset(ve, 0, sizeof(ve));
+
+  for (int i = 0; i < ve_num;i++) {
+     ve[i].src_offset = sizeof(float[4])* i;
+     ve[i].src_format = PIPE_FORMAT_R32G32B32A32_FLOAT;
+  }
+
+   virgl_encoder_create_vertex_elements(&ctx, 2000, ve_num, ve);
+
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, expected_error);
+
+   testvirgl_fini_ctx_cmdbuf(&ctx);
+}
+
+START_TEST(virgl_test_create_vertex_elements_pass)
+{
+   test_vertex_elements(PIPE_MAX_ATTRIBS, 0);
+}
+END_TEST
+
+START_TEST(virgl_test_create_vertex_elements_fail)
+{
+   test_vertex_elements(PIPE_MAX_ATTRIBS + 1, EINVAL);
+}
+END_TEST
+
+static void test_create_shader(enum pipe_shader_type type, int expected_error)
+{
+   struct virgl_context ctx;
+
+   int ret;
+
+   ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
+   ck_assert_int_eq(ret, 0);
+
+   struct pipe_shader_state vs;
+   const char *text =
+       "VERT\n"
+       "DCL IN[0]\n"
+       "DCL IN[1]\n"
+       "DCL OUT[0], POSITION\n"
+       "DCL OUT[1], COLOR\n"
+       "  0: MOV OUT[1], IN[1]\n"
+       "  1: MOV OUT[0], IN[0]\n"
+       "  2: END\n";
+   memset(&vs, 0, sizeof(vs));
+   vs.stream_output.num_outputs = 1;
+   vs.stream_output.stride[0] = 4;
+   vs.stream_output.output[0].num_components = 4;
+   virgl_encode_shader_state(&ctx, 2000, type,
+                             &vs, text);
+
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, expected_error);
+
+   testvirgl_fini_ctx_cmdbuf(&ctx);
+}
+
+START_TEST(virgl_test_create_shader_pass)
+{
+   test_create_shader(PIPE_SHADER_VERTEX, 0);
+}
+END_TEST
+
+START_TEST(virgl_test_create_shader_fail)
+{
+   test_create_shader(PIPE_SHADER_TYPES, EINVAL);
+}
+END_TEST
+
+/* create a resource - clear it to a color, do a transfer */
+START_TEST(virgl_test_clear_texture)
+{
+   struct virgl_context ctx;
+   struct virgl_resource res;
+   union pipe_color_union color;
+   struct virgl_box box;
+   int ret;
+
+   ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
+   ck_assert_int_eq(ret, 0);
+
+   /* init and create simple 2D resource */
+   ret = testvirgl_create_backed_simple_2d_res(&res, 1, 50, 50);
+   ck_assert_int_eq(ret, 0);
+
+   /* attach resource to context */
+   virgl_renderer_ctx_attach_resource(ctx.ctx_id, res.handle);
+
+   /* clear the resource */
+   /* clear buffer to green */
+   color.f[0] = 0.0;
+   color.f[1] = 1.0;
+   color.f[2] = 0.0;
+   color.f[3] = 1.0;
+   box.x = 0;
+   box.y = 0;
+   box.z = 0;
+   box.w = 5;
+   box.h = 1;
+   box.d = 1;
+
+   ret = virgl_encoder_clear_texture(&ctx, res.handle, 0, box, &color);
+   ck_assert_int_eq(ret, 0);
+
+   /* submit the cmd stream */
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, 0);
+
+   /* Check usage with illegal resource */
+   virgl_renderer_ctx_detach_resource(ctx.ctx_id, res.handle);
+
+   ret = virgl_encoder_clear_texture(&ctx, res.handle, 0, box, &color);
+   ck_assert_int_eq(ret, 0);
+
+   /* submit the cmd stream */
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, EINVAL);
+
+   /* cleanup */
+   testvirgl_destroy_backed_res(&res);
+
+   testvirgl_fini_ctx_cmdbuf(&ctx);
+}
+END_TEST
+
+static void test_draw_vbo_unified(uint32_t indirect_handle, uint32_t indirect_draw_count_handle, int expected_error)
+{
+   struct virgl_context ctx;
+   struct virgl_resource res;
+   struct virgl_resource vbo;
+   struct pipe_vertex_buffer vbuf;
+   int ret;
+   int tw = 300, th = 300;
+   int vs_handle, fs_handle;
+   int ctx_handle = 1;
+   union pipe_color_union color;
+
+   struct virgl_surface surf;
+   struct pipe_framebuffer_state fb_state;
+   struct virgl_box box;
+
+   ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
+   ck_assert_int_eq(ret, 0);
+
+   /* init and create simple 2D resource */
+   ret = testvirgl_create_backed_simple_2d_res(&res, 1, tw, th);
+   ck_assert_int_eq(ret, 0);
+
+   /* attach resource to context */
+   virgl_renderer_ctx_attach_resource(ctx.ctx_id, res.handle);
+
+   /* create a surface for the resource */
+   memset(&surf, 0, sizeof(surf));
+   surf.base.format = PIPE_FORMAT_B8G8R8X8_UNORM;
+   surf.handle = ctx_handle++;
+   surf.base.texture = &res.base;
+
+   virgl_encoder_create_surface(&ctx, surf.handle, &res, &surf.base);
+
+   /* set the framebuffer state */
+   fb_state.nr_cbufs = 1;
+   fb_state.zsbuf = NULL;
+   fb_state.cbufs[0] = &surf.base;
+   virgl_encoder_set_framebuffer_state(&ctx, &fb_state);
+
+   /* clear the resource */
+   /* clear buffer to green */
+   color.f[0] = 0.0;
+   color.f[1] = 1.0;
+   color.f[2] = 0.0;
+   color.f[3] = 1.0;
+   virgl_encode_clear(&ctx, PIPE_CLEAR_COLOR0, &color, 0.0, 0);
+
+
+   /* create vbo */
+   ret = testvirgl_create_backed_simple_buffer(&vbo, 2, sizeof(vertices), PIPE_BIND_VERTEX_BUFFER);
+   ck_assert_int_eq(ret, 0);
+   virgl_renderer_ctx_attach_resource(ctx.ctx_id, vbo.handle);
+
+   /* inline write the data to it */
+   box.x = 0;
+   box.y = 0;
+   box.z = 0;
+   box.w = sizeof(vertices);
+   box.h = 1;
+   box.d = 1;
+   virgl_encoder_inline_write(&ctx, &vbo, 0, 0, (struct pipe_box *)&box, &vertices, box.w, 0);
+
+   vbuf.stride = sizeof(struct vertex);
+   vbuf.buffer_offset = 0;
+   vbuf.buffer = &vbo.base;
+   virgl_encoder_set_vertex_buffers(&ctx, 1, &vbuf);
+
+   /* create vertex shader */
+   {
+     struct pipe_shader_state vs;
+     const char *text =
+         "VERT\n"
+         "DCL IN[0]\n"
+         "DCL IN[1]\n"
+         "DCL OUT[0], POSITION\n"
+         "DCL OUT[1], COLOR\n"
+         "  0: MOV OUT[1], IN[1]\n"
+         "  1: MOV OUT[0], IN[0]\n"
+         "  2: END\n";
+     memset(&vs, 0, sizeof(vs));
+     vs_handle = ctx_handle++;
+     virgl_encode_shader_state(&ctx, vs_handle, PIPE_SHADER_VERTEX,
+                               &vs, text);
+     virgl_encode_bind_shader(&ctx, vs_handle, PIPE_SHADER_VERTEX);
+   }
+
+   /* create fragment shader */
+   {
+     struct pipe_shader_state fs;
+     const char *text =
+         "FRAG\n"
+         "DCL IN[0], COLOR, LINEAR\n"
+         "DCL OUT[0], COLOR\n"
+         "  0: MOV OUT[0], IN[0]\n"
+         "  1: END\n";
+     memset(&fs, 0, sizeof(fs));
+     fs_handle = ctx_handle++;
+     virgl_encode_shader_state(&ctx, fs_handle, PIPE_SHADER_FRAGMENT,
+                               &fs, text);
+
+     virgl_encode_bind_shader(&ctx, fs_handle, PIPE_SHADER_FRAGMENT);
+   }
+
+   /* link shader */
+   {
+     uint32_t handles[PIPE_SHADER_TYPES];
+     memset(handles, 0, sizeof(handles));
+     handles[PIPE_SHADER_VERTEX] = vs_handle;
+     handles[PIPE_SHADER_FRAGMENT] = fs_handle;
+     virgl_encode_link_shader(&ctx, handles);
+   }
+
+   /* set blend state */
+   {
+     struct pipe_blend_state blend;
+     int blend_handle = ctx_handle++;
+     memset(&blend, 0, sizeof(blend));
+     blend.rt[0].colormask = PIPE_MASK_RGBA;
+     virgl_encode_blend_state(&ctx, blend_handle, &blend);
+     virgl_encode_bind_object(&ctx, blend_handle, VIRGL_OBJECT_BLEND);
+   }
+
+   /* set depth stencil alpha state */
+   {
+     struct pipe_depth_stencil_alpha_state dsa;
+     int dsa_handle = ctx_handle++;
+     memset(&dsa, 0, sizeof(dsa));
+     dsa.depth.writemask = 1;
+     dsa.depth.func = PIPE_FUNC_LESS;
+     virgl_encode_dsa_state(&ctx, dsa_handle, &dsa);
+     virgl_encode_bind_object(&ctx, dsa_handle, VIRGL_OBJECT_DSA);
+   }
+
+   /* set rasterizer state */
+   {
+     struct pipe_rasterizer_state rasterizer;
+     int rs_handle = ctx_handle++;
+     memset(&rasterizer, 0, sizeof(rasterizer));
+     rasterizer.cull_face = PIPE_FACE_NONE;
+     rasterizer.half_pixel_center = 1;
+     rasterizer.bottom_edge_rule = 1;
+     rasterizer.depth_clip = 1;
+     virgl_encode_rasterizer_state(&ctx, rs_handle, &rasterizer);
+     virgl_encode_bind_object(&ctx, rs_handle, VIRGL_OBJECT_RASTERIZER);
+   }
+
+   /* set viewport state */
+   {
+     struct pipe_viewport_state vp;
+     float znear = 0, zfar = 1.0;
+     float half_w = tw / 2.0f;
+     float half_h = th / 2.0f;
+     float half_d = (zfar - znear) / 2.0f;
+
+     vp.scale[0] = half_w;
+     vp.scale[1] = half_h;
+     vp.scale[2] = half_d;
+
+     vp.translate[0] = half_w + 0;
+     vp.translate[1] = half_h + 0;
+     vp.translate[2] = half_d + znear;
+     virgl_encoder_set_viewport_states(&ctx, 0, 1, &vp);
+   }
+
+   /* draw */
+   {
+     struct pipe_draw_info info;
+     memset(&info, 0, sizeof(info));
+     info.count = 3;
+     info.mode = PIPE_PRIM_TRIANGLES;
+     virgl_encoder_draw_vbo_indirect(&ctx, &info, indirect_handle, indirect_draw_count_handle);
+   }
+
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, expected_error);
+
+   /* cleanup */
+   virgl_renderer_ctx_detach_resource(ctx.ctx_id, res.handle);
+
+   testvirgl_destroy_backed_res(&vbo);
+   testvirgl_destroy_backed_res(&res);
+
+   testvirgl_fini_ctx_cmdbuf(&ctx);
+}
+
+START_TEST(virgl_test_draw_vbo_pass)
+{
+   test_draw_vbo_unified(0, 0, 0);
+}
+END_TEST
+
+START_TEST(virgl_test_draw_vbo_fail_indirect_missing_handle)
+{
+   test_draw_vbo_unified(1000, 2000, EINVAL);
+}
+END_TEST
+
+START_TEST(virgl_test_draw_vbo_fail_not_recoverable)
+{
+   struct virgl_context ctx;
+   struct virgl_resource res;
+   struct virgl_resource vbo;
+   struct pipe_vertex_buffer vbuf;
+   int ret;
+   int tw = 300, th = 300;
+
+   ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
+   ck_assert_int_eq(ret, 0);
+
+   /* init and create simple 2D resource */
+   ret = testvirgl_create_backed_simple_2d_res(&res, 1, tw, th);
+   ck_assert_int_eq(ret, 0);
+
+   /* attach resource to context */
+   virgl_renderer_ctx_attach_resource(ctx.ctx_id, res.handle);
+
+   /* create vbo */
+
+   ret = testvirgl_create_backed_simple_buffer(&vbo, 2, sizeof(vertices), PIPE_BIND_VERTEX_BUFFER);
+   ck_assert_int_eq(ret, 0);
+   virgl_renderer_ctx_attach_resource(ctx.ctx_id, vbo.handle);
+
+   vbuf.stride = sizeof(struct vertex);
+   vbuf.buffer_offset = 0;
+   vbuf.buffer = &vbo.base;
+   virgl_encoder_set_vertex_buffers(&ctx, 1, &vbuf);
+
+   /* create not recoverable state */
+   virgl_renderer_ctx_detach_resource(ctx.ctx_id, vbo.handle);
+
+   /* draw */
+   {
+     struct pipe_draw_info info;
+     memset(&info, 0, sizeof(info));
+     info.count = 3;
+     info.mode = PIPE_PRIM_TRIANGLES;
+     virgl_encoder_draw_vbo_indirect(&ctx, &info, 0, 0);
+   }
+
+   ret = testvirgl_ctx_send_cmdbuf(&ctx);
+   ck_assert_int_eq(ret, ENOTRECOVERABLE);
+
+   /* cleanup */
+   virgl_renderer_ctx_detach_resource(ctx.ctx_id, res.handle);
+
+   testvirgl_destroy_backed_res(&vbo);
+   testvirgl_destroy_backed_res(&res);
+
+   testvirgl_fini_ctx_cmdbuf(&ctx);
+}
+END_TEST
+
 static Suite *virgl_init_suite(void)
 {
   Suite *s;
@@ -1327,8 +1988,22 @@ static Suite *virgl_init_suite(void)
   tcase_add_test(tc_core, virgl_decode_set_constant_buffer);
   tcase_add_test(tc_core, virgl_decode_bind_sampler_states);
   tcase_add_test(tc_core, virgl_test_encode_sampler_view);
+  tcase_add_test(tc_core, virgl_test_encode_sampler_view_fail_layers);
+  tcase_add_test(tc_core, virgl_test_encode_sampler_view_texture_buffer);
   tcase_add_test(tc_core, virgl_test_create_surface_pass);
   tcase_add_test(tc_core, virgl_test_create_surface_fail_format);
+  tcase_add_test(tc_core, virgl_test_create_surface_fail_layers);
+  tcase_add_test(tc_core, virgl_test_create_vertex_elements_pass);
+  tcase_add_test(tc_core, virgl_test_create_vertex_elements_fail);
+  tcase_add_test(tc_core, virgl_test_create_shader_pass);
+  tcase_add_test(tc_core, virgl_test_create_shader_fail);
+  tcase_add_test(tc_core, virgl_test_clear_texture);
+  tcase_add_test(tc_core, virgl_test_draw_vbo_pass);
+  tcase_add_test(tc_core, virgl_test_draw_vbo_fail_indirect_missing_handle);
+  tcase_add_test(tc_core, virgl_test_draw_vbo_fail_not_recoverable);
+  tcase_add_test(tc_core, virgl_test_bind_images_shader_pass);
+  tcase_add_test(tc_core, virgl_test_bind_images_shader_fail_layers);
+  tcase_add_test(tc_core, virgl_test_query);
 
   suite_add_tcase(s, tc_core);
   return s;
