@@ -41,7 +41,11 @@
 START_TEST(virgl_fence_create)
 {
    int ret;
-   ret = testvirgl_init_single_ctx();
+
+   ret = virgl_renderer_create_fence(1, 0);
+   ck_assert_int_eq(ret, EINVAL);
+
+   ret = testvirgl_init_single_ctx(context_flags);
    ck_assert_int_eq(ret, 0);
 
    testvirgl_reset_fence();
@@ -56,7 +60,7 @@ START_TEST(virgl_fence_poll)
 {
    const int target_seqno = 50;
    int ret;
-   ret = testvirgl_init_single_ctx();
+   ret = testvirgl_init_single_ctx(context_flags);
    ck_assert_int_eq(ret, 0);
 
    testvirgl_reset_fence();
@@ -88,7 +92,7 @@ START_TEST(virgl_fence_poll_many)
    int ret;
    int i;
 
-   ret = testvirgl_init_single_ctx();
+   ret = testvirgl_init_single_ctx(context_flags);
    ck_assert_int_eq(ret, 0);
 
    testvirgl_reset_fence();
@@ -147,7 +151,7 @@ START_TEST(virgl_fence_export)
    int fd;
    int ret;
 
-   ret = testvirgl_init_single_ctx();
+   ret = testvirgl_init_single_ctx(context_flags);
    ck_assert_int_eq(ret, 0);
 
    testvirgl_reset_fence();
@@ -177,7 +181,7 @@ START_TEST(virgl_fence_export_signaled)
    int ret;
    int i;
 
-   ret = testvirgl_init_single_ctx();
+   ret = testvirgl_init_single_ctx(context_flags);
    ck_assert_int_eq(ret, 0);
 
    /* when there is no active fence, a signaled fd is always returned */
@@ -219,7 +223,7 @@ START_TEST(virgl_fence_export_invalid)
    int fd;
    int ret;
 
-   ret = testvirgl_init_single_ctx();
+   ret = testvirgl_init_single_ctx(context_flags);
    ck_assert_int_eq(ret, 0);
 
    ret = virgl_renderer_create_fence(target_seqno, 0);
