@@ -688,7 +688,7 @@ static void virgl_test_transfer_inline(enum pipe_texture_target target,
   int elsize = target == 0 ? 1 : 4;
   void *data;
   unsigned size;
-  ret = testvirgl_init_ctx_cmdbuf(&ctx);
+  ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
   ck_assert_int_eq(ret, 0);
 
   get_resource_args(target, invalid, &args, &box, 0, large_flags);
@@ -708,6 +708,10 @@ static void virgl_test_transfer_inline(enum pipe_texture_target target,
   ck_assert_int_eq(ret, invalid ? EINVAL : 0);
   virgl_renderer_ctx_detach_resource(ctx.ctx_id, res.handle);
 
+  virgl_encoder_inline_write(&ctx, &res, 0, 0, (struct pipe_box *)&box, data, box.width * elsize, 0);
+  ret = testvirgl_ctx_send_cmdbuf(&ctx);
+  ck_assert_int_eq(ret, EINVAL);
+
   virgl_renderer_resource_unref(res.handle);
   testvirgl_fini_ctx_cmdbuf(&ctx);
   free(data);
@@ -739,7 +743,7 @@ START_TEST(virgl_test_transfer_to_staging_without_iov_fails)
   struct pipe_box box = {.width = bufsize, .height = 1, .depth = 1};
   int ret;
 
-  ret = testvirgl_init_ctx_cmdbuf(&ctx);
+  ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
   ck_assert_int_eq(ret, 0);
 
   ret = testvirgl_create_unbacked_simple_buffer(&res, 1, bufsize, VIRGL_BIND_STAGING);
@@ -766,7 +770,7 @@ START_TEST(virgl_test_transfer_to_staging_with_iov_succeeds)
   struct pipe_box box = {.width = bufsize, .height = 1, .depth = 1};
   int ret;
 
-  ret = testvirgl_init_ctx_cmdbuf(&ctx);
+  ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
   ck_assert_int_eq(ret, 0);
 
   ret = testvirgl_create_backed_simple_buffer(&res, 1, bufsize, VIRGL_BIND_STAGING);
@@ -795,7 +799,7 @@ START_TEST(virgl_test_copy_transfer_from_staging_without_iov_fails)
   struct pipe_box box = {.width = bufsize, .height = 1, .depth = 1};
   int ret;
 
-  ret = testvirgl_init_ctx_cmdbuf(&ctx);
+  ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
   ck_assert_int_eq(ret, 0);
 
   ret = testvirgl_create_unbacked_simple_buffer(&src_res, 1, bufsize, VIRGL_BIND_STAGING);
@@ -830,7 +834,7 @@ START_TEST(virgl_test_copy_transfer_from_staging_with_iov_succeeds)
   struct pipe_box box = {.width = bufsize, .height = 1, .depth = 1};
   int ret;
 
-  ret = testvirgl_init_ctx_cmdbuf(&ctx);
+  ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
   ck_assert_int_eq(ret, 0);
 
   ret = testvirgl_create_backed_simple_buffer(&src_res, 1, bufsize, VIRGL_BIND_STAGING);
@@ -849,6 +853,11 @@ START_TEST(virgl_test_copy_transfer_from_staging_with_iov_succeeds)
 
   virgl_renderer_ctx_detach_resource(ctx.ctx_id, src_res.handle);
   virgl_renderer_ctx_detach_resource(ctx.ctx_id, dst_res.handle);
+
+  virgl_encoder_copy_transfer(&ctx, &dst_res, 0, 0, &box, &src_res, 0, synchronized);
+  ret = testvirgl_ctx_send_cmdbuf(&ctx);
+  ck_assert_int_eq(ret, EINVAL);
+
   testvirgl_destroy_backed_res(&src_res);
   testvirgl_destroy_backed_res(&dst_res);
   testvirgl_fini_ctx_cmdbuf(&ctx);
@@ -865,7 +874,7 @@ START_TEST(virgl_test_copy_transfer_to_staging_without_iov_fails)
   struct pipe_box box = {.width = bufsize, .height = 1, .depth = 1};
   int ret;
 
-  ret = testvirgl_init_ctx_cmdbuf(&ctx);
+  ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
   ck_assert_int_eq(ret, 0);
 
   ret = testvirgl_create_backed_simple_buffer(&src_res, 1, bufsize, VIRGL_BIND_STAGING);
@@ -899,7 +908,7 @@ START_TEST(virgl_test_copy_transfer_to_staging_with_iov_succeeds)
   struct pipe_box box = {.width = bufsize, .height = 1, .depth = 1};
   int ret;
 
-  ret = testvirgl_init_ctx_cmdbuf(&ctx);
+  ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
   ck_assert_int_eq(ret, 0);
 
   ret = testvirgl_create_backed_simple_buffer(&src_res, 1, bufsize, VIRGL_BIND_STAGING);
@@ -933,7 +942,7 @@ START_TEST(virgl_test_transfer_near_res_bounds_with_stride_succeeds)
   struct pipe_box box = {.x = 2, .y = 1, .z = 0, .width = 2, .height = 2, .depth = 1};
   int ret;
 
-  ret = testvirgl_init_ctx_cmdbuf(&ctx);
+  ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
   ck_assert_int_eq(ret, 0);
 
   ret = testvirgl_create_backed_simple_2d_res(&res, 1, res_width, res_height);
@@ -959,7 +968,7 @@ START_TEST(test_vrend_host_backed_memory_no_data_leak)
 
    struct virgl_context ctx = {0};
 
-   int ret = testvirgl_init_ctx_cmdbuf(&ctx);
+   int ret = testvirgl_init_ctx_cmdbuf(&ctx, context_flags);
 
    struct virgl_renderer_resource_create_args res;
    res.handle = 0x400;
