@@ -205,7 +205,7 @@ static void virgl_emit_shader_streamout(struct virgl_context *ctx,
                                         const struct pipe_stream_output_info *so_info)
 {
    int num_outputs = 0;
-   uint i;
+   uint32_t i;
    uint32_t tmp;
 
    if (so_info)
@@ -321,7 +321,7 @@ int virgl_encoder_set_framebuffer_state(struct virgl_context *ctx,
                                        const struct pipe_framebuffer_state *state)
 {
    struct virgl_surface *zsurf = (struct virgl_surface *)state->zsbuf;
-   uint i;
+   uint32_t i;
 
    virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_SET_FRAMEBUFFER_STATE, 0, VIRGL_SET_FRAMEBUFFER_STATE_SIZE(state->nr_cbufs)));
    virgl_encoder_write_dword(ctx->cbuf, state->nr_cbufs);
@@ -356,7 +356,7 @@ int virgl_encoder_create_vertex_elements(struct virgl_context *ctx,
                                         unsigned num_elements,
                                         const struct pipe_vertex_element *element)
 {
-   uint i;
+   uint32_t i;
    virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_CREATE_OBJECT, VIRGL_OBJECT_VERTEX_ELEMENTS, VIRGL_OBJ_VERTEX_ELEMENTS_SIZE(num_elements)));
    virgl_encoder_write_dword(ctx->cbuf, handle);
    for (i = 0; i < num_elements; i++) {
@@ -372,7 +372,7 @@ int virgl_encoder_set_vertex_buffers(struct virgl_context *ctx,
                                     unsigned num_buffers,
                                     const struct pipe_vertex_buffer *buffers)
 {
-   uint i;
+   uint32_t i;
    virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_SET_VERTEX_BUFFERS, 0, VIRGL_SET_VERTEX_BUFFERS_SIZE(num_buffers)));
    for (i = 0; i < num_buffers; i++) {
       struct virgl_resource *res = (struct virgl_resource *)buffers[i].buffer;
@@ -402,6 +402,14 @@ int virgl_encoder_set_index_buffer(struct virgl_context *ctx,
 
 int virgl_encoder_draw_vbo(struct virgl_context *ctx,
                           const struct pipe_draw_info *info)
+{
+   return virgl_encoder_draw_vbo_indirect(ctx, info, 0, 0);
+}
+
+int virgl_encoder_draw_vbo_indirect(struct virgl_context *ctx,
+                                const struct pipe_draw_info *info,
+                                uint32_t indirect_handle,
+                                uint32_t indirect_draw_count_handle)
 {
    virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_DRAW_VBO, 0, VIRGL_DRAW_VBO_SIZE));
    virgl_encoder_write_dword(ctx->cbuf, info->start);
@@ -415,6 +423,16 @@ int virgl_encoder_draw_vbo(struct virgl_context *ctx,
    virgl_encoder_write_dword(ctx->cbuf, info->restart_index);
    virgl_encoder_write_dword(ctx->cbuf, info->min_index);
    virgl_encoder_write_dword(ctx->cbuf, info->max_index);
+
+   if (indirect_handle != 0) {
+      virgl_encoder_write_dword(ctx->cbuf, indirect_handle);
+      virgl_encoder_write_dword(ctx->cbuf, info->indirect.offset);
+      virgl_encoder_write_dword(ctx->cbuf, info->indirect.stride);
+      virgl_encoder_write_dword(ctx->cbuf, info->indirect.draw_count);
+      virgl_encoder_write_dword(ctx->cbuf, info->indirect.indirect_draw_count_offset);
+      virgl_encoder_write_dword(ctx->cbuf, indirect_draw_count_handle);
+   }
+
    if (info->count_from_stream_output)
       virgl_encoder_write_dword(ctx->cbuf, info->count_from_stream_output->buffer_size);
    else
@@ -442,6 +460,31 @@ int virgl_encoder_create_surface(struct virgl_context *ctx,
    return 0;
 }
 
+int virgl_encoder_clear_texture(struct virgl_context *ctx,
+                                 uint32_t handle,
+                                 uint32_t level,
+                                 struct virgl_box box,
+                                 const union pipe_color_union *color)
+{
+   virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_CLEAR_TEXTURE, 0, VIRGL_CLEAR_TEXTURE_SIZE));
+   virgl_encoder_write_dword(ctx->cbuf, handle);
+   virgl_encoder_write_dword(ctx->cbuf, level);
+
+   virgl_encoder_write_dword(ctx->cbuf, box.x);
+   virgl_encoder_write_dword(ctx->cbuf, box.y);
+   virgl_encoder_write_dword(ctx->cbuf, box.z);
+   virgl_encoder_write_dword(ctx->cbuf, box.w);
+   virgl_encoder_write_dword(ctx->cbuf, box.h);
+   virgl_encoder_write_dword(ctx->cbuf, box.d);
+
+   virgl_encoder_write_dword(ctx->cbuf, color->ui[0]);
+   virgl_encoder_write_dword(ctx->cbuf, color->ui[1]);
+   virgl_encoder_write_dword(ctx->cbuf, color->ui[2]);
+   virgl_encoder_write_dword(ctx->cbuf, color->ui[3]);
+
+   return 0;
+}
+
 int virgl_encoder_create_so_target(struct virgl_context *ctx,
                                   uint32_t handle,
                                   struct virgl_resource *res,
@@ -653,7 +696,7 @@ int virgl_encode_sampler_view(struct virgl_context *ctx,
    virgl_encoder_write_dword(ctx->cbuf, handle);
    virgl_encoder_write_res(ctx, res);
    virgl_encoder_write_dword(ctx->cbuf, state->format);
-   if (1) {//TODOres->u.b.target == PIPE_BUFFER) {
+   if (res->base.target == PIPE_BUFFER) {
       virgl_encoder_write_dword(ctx->cbuf, state->u.buf.first_element);
       virgl_encoder_write_dword(ctx->cbuf, state->u.buf.last_element);
    } else {
@@ -674,7 +717,7 @@ int virgl_encode_set_sampler_views(struct virgl_context *ctx,
                                   uint32_t num_views,
                                   struct virgl_sampler_view **views)
 {
-   uint i;
+   uint32_t i;
    virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_SET_SAMPLER_VIEWS, 0, VIRGL_SET_SAMPLER_VIEWS_SIZE(num_views)));
    virgl_encoder_write_dword(ctx->cbuf, shader_type);
    virgl_encoder_write_dword(ctx->cbuf, start_slot);
@@ -691,7 +734,7 @@ int virgl_encode_bind_sampler_states(struct virgl_context *ctx,
                                     uint32_t num_handles,
                                     uint32_t *handles)
 {
-   uint i;
+   uint32_t i;
    virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_BIND_SAMPLER_STATES, 0, VIRGL_BIND_SAMPLER_STATES(num_handles)));
    virgl_encoder_write_dword(ctx->cbuf, shader_type);
    virgl_encoder_write_dword(ctx->cbuf, start_slot);
@@ -730,6 +773,28 @@ int virgl_encoder_set_uniform_buffer(struct virgl_context *ctx,
    return 0;
 }
 
+int virgl_encoder_set_shader_images(struct virgl_context *ctx,
+                                     uint32_t shader_type,
+                                     uint32_t start_slot,
+                                     uint32_t num_images,
+                                    struct vrend_image_view *images,
+                                    uint32_t handle)
+{
+   virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_SET_SHADER_IMAGES, 0, VIRGL_SET_SHADER_IMAGE_SIZE(num_images)));
+   virgl_encoder_write_dword(ctx->cbuf, shader_type);
+   virgl_encoder_write_dword(ctx->cbuf, start_slot);
+
+   for (uint32_t i = 0; i < num_images; i++) {
+      virgl_encoder_write_dword(ctx->cbuf, images[i].format);
+      virgl_encoder_write_dword(ctx->cbuf, images[i].access);
+      virgl_encoder_write_dword(ctx->cbuf, images[i].u.buf.offset);
+      virgl_encoder_write_dword(ctx->cbuf, images[i].u.buf.size);
+
+      virgl_encoder_write_dword(ctx->cbuf, handle);
+   }
+
+   return 0;
+}
 
 int virgl_encoder_set_stencil_ref(struct virgl_context *ctx,
                                  const struct pipe_stencil_ref *ref)
@@ -856,7 +921,7 @@ int virgl_encode_blit(struct virgl_context *ctx,
 
 int virgl_encoder_create_query(struct virgl_context *ctx,
                               uint32_t handle,
-                              uint query_type,
+                              uint32_t query_type,
                               struct virgl_resource *res,
                               uint32_t offset)
 {
@@ -893,9 +958,25 @@ int virgl_encoder_get_query_result(struct virgl_context *ctx,
    return 0;
 }
 
+int virgl_encoder_get_query_result_qbo(struct virgl_context *ctx,
+                                      uint32_t handle, uint32_t handle_qbo,
+                                      boolean wait_qbo, uint32_t result_type,
+                                      uint32_t type_qbo, uint32_t qbo_index)
+{
+   virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_GET_QUERY_RESULT_QBO, 0, 6));
+   virgl_encoder_write_dword(ctx->cbuf, handle);
+   virgl_encoder_write_dword(ctx->cbuf, handle_qbo);
+   virgl_encoder_write_dword(ctx->cbuf, wait_qbo ? 1 : 0);
+   virgl_encoder_write_dword(ctx->cbuf, result_type);
+   virgl_encoder_write_dword(ctx->cbuf, type_qbo);
+   virgl_encoder_write_dword(ctx->cbuf, qbo_index);
+
+   return 0;
+}
+
 int virgl_encoder_render_condition(struct virgl_context *ctx,
                                   uint32_t handle, boolean condition,
-                                  uint mode)
+                                  uint32_t mode)
 {
    virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_SET_RENDER_CONDITION, 0, VIRGL_RENDER_CONDITION_SIZE));
    virgl_encoder_write_dword(ctx->cbuf, handle);
@@ -909,7 +990,7 @@ int virgl_encoder_set_so_targets(struct virgl_context *ctx,
                                 struct pipe_stream_output_target **targets,
                                 unsigned append_bitmask)
 {
-   uint i;
+   uint32_t i;
 
    virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_SET_STREAMOUT_TARGETS, 0, num_targets + 1));
    virgl_encoder_write_dword(ctx->cbuf, append_bitmask);
@@ -962,3 +1043,58 @@ int virgl_encode_bind_shader(struct virgl_context *ctx,
    virgl_encoder_write_dword(ctx->cbuf, type);
    return 0;
 }
+
+int virgl_encode_set_shader_images(struct virgl_context *ctx,
+                                   uint32_t shader_type,
+                                   uint32_t start_slot,
+                                   uint32_t num_images,
+                                   const struct virgl_shader_image *images)
+{
+   virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_SET_SHADER_IMAGES, 0,
+                                                 VIRGL_SET_SHADER_IMAGE_SIZE(num_images)));
+   virgl_encoder_write_dword(ctx->cbuf, shader_type);
+   virgl_encoder_write_dword(ctx->cbuf, start_slot);
+   for (uint32_t i = 0; i < num_images; ++i) {
+      virgl_encoder_write_dword(ctx->cbuf, images[i].format);
+      virgl_encoder_write_dword(ctx->cbuf, images[i].access);
+      virgl_encoder_write_dword(ctx->cbuf, images[i].layer_offset);
+      virgl_encoder_write_dword(ctx->cbuf, images[i].level_size);
+      virgl_encoder_write_dword(ctx->cbuf, images[i].handle);
+   }
+   return 0;
+}
+
+int virgl_encode_set_shader_buffers(struct virgl_context *ctx,
+                                    uint32_t shader_type,
+                                    uint32_t start_slot,
+                                    uint32_t num_buffers,
+                                    const struct virgl_shader_buffer *buffers)
+{
+   virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_SET_SHADER_BUFFERS, 0,
+                                                 VIRGL_SET_SHADER_BUFFER_SIZE(num_buffers)));
+   virgl_encoder_write_dword(ctx->cbuf, shader_type);
+   virgl_encoder_write_dword(ctx->cbuf, start_slot);
+   for (uint32_t i = 0; i < num_buffers; ++i) {
+      virgl_encoder_write_dword(ctx->cbuf, buffers[i].offset);
+      virgl_encoder_write_dword(ctx->cbuf, buffers[i].buf_len);
+      virgl_encoder_write_dword(ctx->cbuf, buffers[i].handle);
+   }
+   return 0;
+}
+
+int virgl_encode_simple_launch_grid(struct virgl_context *ctx, uint32_t grid[3])
+{
+   virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_LAUNCH_GRID, 0, VIRGL_LAUNCH_GRID_SIZE));
+
+   for (int i = 0; i < 3; ++i) /* block */
+      virgl_encoder_write_dword(ctx->cbuf, 0);
+
+   for (int i = 0; i < 3; ++i)
+      virgl_encoder_write_dword(ctx->cbuf, grid[i]);
+
+   virgl_encoder_write_dword(ctx->cbuf, 0);/* indirect handle */
+   virgl_encoder_write_dword(ctx->cbuf, 0);/* indirect offset */
+
+   return 0;
+}
+
